import markdown


def html(text):
    return markdown.markdown(text=text, extensions=["markdown.extensions.tables"])


class directionresult:
    def __init__(self):
        self.list = ["|market|code|type|direction|", "|---|---|---|---|"]
        self.up = 0
        self.down = 0
        self.count = 0

    def __add__(self, other):
        self.count += 1
        if other[3] == "UP":
            self.up += 1
        else:
            self.down += 1
        self.list.append("|".join(other))

    def tomarkdown(self):
        return ["## Total:" + str(self.count), "### UP:" + str(self.up), "### DOWN:" + str(self.down),
                "\r\n".join(self.list)]

    def tohtml(self):
        return html("\r\n".join(self.tomarkdown()))



# -*- coding: UTF-8 -*-
"""
TrHub
=====
"""
import redis
import time
import pandas as pd
import format as fm
import trhub.utils as utils
import trhub.utils.mkdown as mkdown


@utils.singleton
class TuRingDataHub:
    def __init__(self, host="localhost", port=6379, db=0, password=""):
        self.host = host
        self.port = port
        self.db = db
        self.password = password
        self.client = None
        self.isinit = False
        self.init(host, port, db, password)

    def init(self, host="localhost", port=6379, db=0, password=""):
        """
            初始化redis
        """
        if self.isinit:
            return self.client
        else:
            self.client = redis.StrictRedis(host, port, db, password)
            self.isinit = True
        return self.client

    def getdays(self, market="SSE", code="000001", type="INDEX", begintime=(time.time() - (60 * 60 * 24 * 500)) * 1000,
                endtime=time.time() * 1000, num=None):
        """
            通过 市场,代码,类型 得到日K数据,默认500天前
            Key:
                SS.DAYS.${CODE}.DATE.${MARKET}.${TYPE}
        """
        key = ".".join(("SS", "DAYS", code, "DATE", market, type))
        if num is not None:
            result = self.client.zrange(key, -num, -1)
        else:
            result = self.client.zrangebyscore(key, begintime, endtime)
        return pd.DataFrame(map(lambda x: pd.Series(fm.kline(x.split(","))), result))

    def gettradecode(self, market="SSE", type="A_STOCK", day=None):
        """
            通过 市场,类型,日期 得到交易日的 交易品种, 不提供日期,默认 最新交易日
            Key:
                SS.MARKET.CODE.${DATE}.${MARKET}.${TYPE}
        """

        if day is None:
            day = self.getlasttradeday(market=market, type=type)["date"][0]

        key = ".".join(("SS", "MARKET", "CODE", day, market, type))
        result = self.client.zrange(key, 0, -1, withscores=True)

        def tuple2array(i, i2):
            item = i.split(",")
            item.append(i2)
            return item

        return pd.DataFrame(map(lambda x: pd.Series(fm.code(tuple2array(x[0], x[1]))), result))

    def gettradedays(self, market="SSE", type="A_STOCK", begin=0, end=-1):
        """
            通过 市场,类型 得到 交易日
            Key:
                SS.TRADETIME.DAYS.DATE.${MARKET}.${TYPE}
        """
        key = ".".join(("SS", "TRADETIME", "DAYS", "DATE", market, type))
        result = self.client.zrange(key, begin, end, withscores=True)
        return pd.DataFrame(map(lambda x: pd.Series(fm.tradeday([x[0], x[1]])), result))

    def getlasttradeday(self, market="SSE", type="A_STOCK"):
        """
            通过 市场,类型 得到 最后交易日
            Key:
                SS.TRADETIME.DAYS.DATE.${MARKET}.${TYPE}
        """
        return self.gettradedays(market=market, type=type, begin=-1, end=-1)

    def saveresult(self, list, direction, name, market, type, date="20160918"):
        """
            把结果保存到数据库中
            Key:
                SE.QUANT.${NAME}.${DATE}.${MARKET}.${TYPE}
        """
        name = name.upper()
        market = market.upper()
        type = type.upper()
        key = ".".join(("SE", "QUANT", name + "(" + direction + ")", date, market, type))
        self.client.sadd(key, *list)

    def saveupresult(self, list, name, market, type, date):
        """
            存储涨的股票代码列表
        """
        self.saveresult(list, "UP", name, market, type, date)

    def savedownresult(self, list, name, market, type, date):
        """
            存储跌的股票代码列表
        """
        self.saveresult(list, "DOWN", name, market, type, date)

    def saveinspectresult(self, up, down, upright, downright, name, type, date):
        """
        存储验证结果
        key: SE.QUANT.${NAME}.${DATE}.${TYPE}
        :param data:验证的涨跌数据
        :param name:策略名
        :param type:市场类型
        :param date:日期
        """
        name = name.upper()
        type = type.upper()
        date = date.upper()
        key = '.'.join(('SE', 'INSPECT', name, date, type))
        data = {'uplength': up, 'downlength': down, 'upright': upright, 'downright': downright}
        self.client.sadd(key, data)

    def getinspectresult(self, name='DEMO', type='A_STOCK', date='20160918'):
        """
        得到存储在redis中的结果
        key: SE.QUANT.${NAME}.${DATE}.${TYPE}
        :param name:策略名
        :param type:市场类型
        :param date:日期
        :return: 结果集合
        """
        name = name.upper()
        type = type.upper()
        key = '.'.join(('SE', 'INSPECT', name, date, type))
        return self.client.smembers(key)


    def getresult(self, direction="UP", name="DEMO", market="SZSE", type="A_STOCK", date="20160918"):
        """
            得到存储在redis中的结果
            Key:
                SE.QUANT.${NAME}.${DATE}.${MARKET}.${TYPE}
        """
        name = name.upper()
        market = market.upper()
        type = type.upper()
        key = ".".join(("SE", "QUANT", name + "(" + direction + ")", date, market, type))
        return self.client.smembers(key)

    def getupresult(self, name="DEMO", market="SZSE", type="A_STOCK", date="20160918"):
        return self.getresult("UP", name, market, type, date)

    def getdownresult(self, name="DEMO", market="SZSE", type="A_STOCK", date="20160918"):
        return self.getresult("DOWN", name, market, type, date)

    def getresulthtml(self, name="DEMO", market="SZSE", type="A_STOCK", date="20160918"):
        md = mkdown.directionresult()
        upresult = self.getupresult(name, market, type, date)
        for item in upresult:
            md + [market, item, type, "UP"]
        downresult = self.getdownresult(name, market, type, date)
        for item in downresult:
            md + [market, item, type, "DOWN"]
        return md.tohtml()


DHub = TuRingDataHub



from email.message import Message
from smtplib import SMTP
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart


class myemail:
    def __init__(self, host, username, password):
        self.host = host
        self.username = username
        self.password = password
        self.sm = SMTP(host=self.host)
        self.sm.login(self.username, self.password)

    def send(self, title, html, to):
        msgAlternative = MIMEMultipart('alternative')
        msg = MIMEText(
            "<html><head><style>table {width: 100%}th, td {border: 1px solid #000000;text-align: center;}</style></head><body>" + html + "</body></html>",
            'html', _charset="utf-8")
        msgAlternative["Subject"] = title
        msgAlternative["From"] = self.username
        if type(to) is not list:
            to = [to]
        msg["To"] = ",".join(to)
        msgAlternative.attach(msg)
        # msg.set_payload(html)
        self.sm.sendmail(self.username, to, msgAlternative.as_string())



# -*- coding: UTF-8 -*-
# import trhub.utils as utils
# @utils.singleton

class Trade:
    def __init__(self, name, money):
        self.name = name
        self.records = []
        self.money = money
        self.trades = {"up": [], "down": []}

    def opentrade(self, direction="up", time=None, money=None):
        if self.money >= money:
            self.trades[direction].append({"time": time, "money": money})
            self.money -= money
            # else:
            #     print "没有足够的钱咯~"

    def closetrade(self, direction="up", time=None, money=None):
        if len(self.trades[direction]) != 0:
            trade = self.trades[direction].pop()
            self.records.append(
                {"direction": direction, "open": trade["money"], "close": money, "opentime": trade["time"],
                 "closetime": time})
            if direction is "up":
                self.money += money
            else:
                self.money += (trade["money"] * 2 - money)

    def open(self, direction="up", hand=1, money=None, time=None):
        if money is not None:
            for i in range(0, hand):
                self.opentrade(direction, time, money)

    def close(self, direction="up", hand=1, money=None, time=None):
        if money is not None:
            for i in range(0, hand):
                self.closetrade(direction, time, money)

    def buy(self, hand=1, money=None, time=None):
        self.open("up", hand, money, time)

    def sell(self, hand=1, money=None, time=None):
        self.open("down", hand, money, time)

    def closebuy(self, hand=1, money=None, time=None):
        self.close("up", hand, money, time)

    def closesell(self, hand=1, money=None, time=None):
        self.close("down", hand, money, time)

    def myvalue(self, money=None):
        mymoney = self.money + len(self.trades["up"]) * money;
        for i in range(0, len(self.trades["down"])):
            item = self.trades["down"][i]
            # 看空算法 现今价值  = 成本+(成本-现价)
            # 根据算数优化, 现今价值 = 成本*2 - 现价
            mymoney += (item["money"] * 2 - money)
            # return len(self.trades["up"]) * money + len(self.trades["down"]) * money + self.money
        return mymoney


def singleton(cls):
    instances = {}

    def _singleton(*args, **kw):
        if cls not in instances:
            instances[cls] = cls(*args, **kw)
        return instances[cls]

    return _singleton


#!/usr/bin/python
# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages

setup(
    name="trhub",
    version='0.1.3',
    keywords=("图灵", "数据中心"),
    description=("图灵数据中心"),
    license="MIT License",
    install_requires=["pandas", "redis"],
    author="iamee",
    author_email="admin@cavacn.com",
    packages=find_packages(),
    platforms="any"

)
